import os
from pathlib import Path
from xd_cwl_utils.config import config

# IDEA: Using Path class, it might be better to generate the get methods by getting longest path, then using Path.parents
# for parent directories. More maintainable if we change path structure. Maybe not.

# Misc

def get_inputs_schema_template():

    schema_template_path = Path.cwd() / 'tests/test_files/schema_salad/inputs_schema_template.yml'

    return schema_template_path

# cwl-tools

def get_tool_version_dir(tool_name, tool_version):
    version_dir = config[os.environ['CONFIG_KEY']]['cwl_tool_dir'] / tool_name / tool_version
    return version_dir

def get_tool_dir(tool_name, tool_version, subtool_name=None):
    tool_version_dir = get_tool_version_dir(tool_name, tool_version)
    if subtool_name:
        tool_dir = tool_version_dir / f"{tool_name}_{subtool_name}"
    else:
        tool_dir = tool_version_dir / tool_name
    return tool_dir

def get_cwl_tool(tool_name, tool_version, subtool_name=None):
    tool_dir = get_tool_dir(tool_name, tool_version, subtool_name=subtool_name)

    if subtool_name:
        cwl_tool_path = tool_dir / f"{tool_name}-{subtool_name}.cwl"
    else:
        cwl_tool_path = tool_dir / f"{tool_name}.cwl"
    return cwl_tool_path


def get_cwl_tool_metadata(tool_name, tool_version, subtool_name=None, parent=False):
    version_dir = get_tool_version_dir(tool_name, tool_version)

    if parent:
        cwl_tool_metadata_path = version_dir / 'common' / f"{tool_name}-metadata.yaml"
    else:
        cwl_tool_metadata_path = get_metadata_path(get_cwl_tool(tool_name, tool_version, subtool_name=subtool_name))
    return cwl_tool_metadata_path


def get_tool_inputs_dir(tool_name, tool_version, subtool_name=None):
    cwl_tool_dir = get_tool_dir(tool_name, tool_version, subtool_name=subtool_name)
    instances_dir = cwl_tool_dir / 'instances'
    return instances_dir



def get_tool_instance_path(tool_name, tool_version, input_hash, subtool_name=None):

    cwl_tool_inst_dir = get_tool_inputs_dir(tool_name, tool_version, subtool_name=subtool_name)
    inputs_path = cwl_tool_inst_dir / f"{input_hash}.yaml"

    return inputs_path

def get_tool_args_from_path(cwl_tool_path):

    if not isinstance(cwl_tool_path, Path):
        cwl_tool_path = Path(cwl_tool_path)
    parents = cwl_tool_path.parents
    print('Parents ', parents)
    return

# cwl-scripts

def get_script_version_dir(group_name, project_name, version):
    script_ver_dir = config[os.environ['CONFIG_KEY']]['cwl_script_dir'] / group_name / project_name / version
    return script_ver_dir

def get_cwl_script(group_name, project_name, version, script_name):
    script_ver_dir = get_script_version_dir(group_name, project_name, version)
    script_path = script_ver_dir / script_name / f"{script_name}.cwl"
    return script_path

def get_script_metadata(group_name, project_name, version, script_name):
    script_ver_dir = get_script_version_dir(group_name, project_name, version)
    script_metadata_path = script_ver_dir / script_name / f"{script_name}-metadata.cwl"
    return script_metadata_path


def get_script_inputs():
    raise NotImplementedError


# cwl-workflows

def get_workflow_version_dir(group_name, project_name, version):
    workflow_ver_dir = config[os.environ['CONFIG_KEY']]['cwl_workflows_dir'] / group_name / project_name / version
    return workflow_ver_dir

def get_cwl_workflow(group_name, project_name, version, workflow_name):
    workflow_ver_dir = get_workflow_version_dir(group_name, project_name, version)
    workflow_path = workflow_ver_dir / f"{workflow_name}.cwl"
    return workflow_path

def get_workflow_metadata(group_name, project_name, version, workflow_name):
    workflow_ver_dir = get_workflow_version_dir(group_name, project_name, version)
    workflow_metadata_path = workflow_ver_dir / f"{workflow_name}-metadata.yaml"
    return workflow_metadata_path

def get_workflow_inputs():
    raise NotImplementedError



# helpers

def get_relative_path(full_path, base_path=Path.cwd()):

    return full_path.relative_to(base_path)

def get_metadata_path(cwl_path):
    if not isinstance(cwl_path, Path):
        cwl_path = Path(cwl_path)
    path_dir = cwl_path.parent
    metafile_name = f"{cwl_path.stem}-metadata.yaml"
    metadata_path = path_dir / metafile_name
    return metadata_path




from urllib.parse import urlparse

def uri_name(input_uri):
    # return the part after the '#'
    d = urlparse(input_uri)
    if d.fragment:
        return d.fragment
    return d.path.split("#")[-1]



from xd_cwl_utils.helpers.get_paths import get_cwl_tool, get_tool_instance_path, get_tool_dir
from xd_cwl_utils.classes.schema_salad.schema_salad import InputsSchema

def validate_inputs_for_instance(tool_name, tool_version, instance_hash, subtool_name=None):
    cwl_path = get_cwl_tool(tool_name, tool_version, subtool_name=subtool_name)
    inputs_schema = InputsSchema(cwl_path)
    document_path = get_tool_instance_path(tool_name, tool_version, instance_hash, subtool_name=subtool_name)
    inputs_schema.validate_inputs(document_path)  # Will raise error if not valid.
    return


def validate_all_inputs_for_tool_version(tool_name, tool_version, subtool_name=None):
    version_dir = get_tool_dir(tool_name, tool_version, subtool_name=subtool_name)

    raise NotImplementedError

def validate_all_instance_inputs():
    raise NotImplementedError


#
# * This file is subject to the terms and conditions defined in
# * file 'LICENSE.txt', which is part of this source code package.

from ...classes.metadata.metadata_base import object_attributes
from hashlib import md5
import uuid


def _mk_hashes(arg1, *args):
    arg1 = str(arg1)
    hashes = [md5(arg1.encode(encoding='utf-8')).hexdigest()]
    for arg in args:
        arg = str(arg)
        hashes.append(md5(arg.encode(encoding='utf-8')).hexdigest())
    return hashes

def mk_tool_identifier(name, version, start=0):
    name_hash, version_hash = _mk_hashes(name, version)
    identifier = f"TL_{name_hash[start:start+6]}.{version_hash[:2]}"
    return identifier

def mk_tool_instance_identifier(tool_identifier):
    uuid_string = uuid.uuid4().hex[:4]
    return f"{tool_identifier}.{uuid_string}"

def mk_subtool_identifier():
    raise NotImplementedError

def mk_script_identifier():
    raise NotImplementedError

def mk_workflow_identifier():
    raise NotImplementedError


def is_attr_empty(attribute):
    # Need to check for lists.
    if isinstance(attribute, list):
        is_empty = True
        for item in attribute:
            if not is_attr_empty(item):
                is_empty = False
                break
    elif isinstance(attribute, object_attributes):
        is_empty = attribute.is_empty()
    else:
        if attribute:
            is_empty = False
        else:
            is_empty = True
    return is_empty


class NameSoftwareVersionMixin:
    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, new_name):
        if not new_name:
            raise ValueError(f"'name'' must be set.")
        self._name = new_name

    @property
    def softwareVersion(self):
        return self._softwareVersion

    @softwareVersion.setter
    def softwareVersion(self, new_softwareVersion):

        if not new_softwareVersion:  # softwareVersion not defined. Check in parentMetadata
            raise ValueError(f"'softwareVersion must be set.")
        self._softwareVersion = new_softwareVersion
        return




from tests.test_base import TestBase
from pathlib import Path
from xd_cwl_utils.helpers.get_paths import get_cwl_tool_metadata
from xd_cwl_utils.validate_metadata import metadata_validator_factory
from xd_cwl_utils.classes.metadata.tool_metadata import ToolMetadata, ParentToolMetadata, SubtoolMetadata


class TestValidateMetadata(TestBase):

    def test_validate_tool_metadata(self):
        validate_tool_metadata = metadata_validator_factory(ToolMetadata)
        metadata_path = get_cwl_tool_metadata('sort', '8.25')
        validate_tool_metadata(metadata_path)
        return

    def test_validate_tool_metadata_fail(self):
        validate_tool_metadata = metadata_validator_factory(ToolMetadata)
        metadata_path = Path().cwd() / 'tests/test_files/bad_gawk-metadata.yaml'
        with self.assertRaises(ValueError):
            validate_tool_metadata(metadata_path)
        return

    def test_validate_parent_tool_metadata(self):
        validate_parent_metadata = metadata_validator_factory(ParentToolMetadata)
        metadata_path = get_cwl_tool_metadata('STAR', '2.5', parent=True)
        validate_parent_metadata(metadata_path)

    def test_validate_parent_tool_metadata_fail(self):
        validate_parent_metadata = metadata_validator_factory(ParentToolMetadata)
        metadata_path = Path.cwd() / ('tests/test_files/test_content/cwl_tools_bad/samtools/1.3/common/bad_samtools-metadata.yaml')
        with self.assertRaises(AttributeError):
            validate_parent_metadata(metadata_path)
        return

    def test_validate_subtool_metadata(self):
        validate_subtool_metadata = metadata_validator_factory(SubtoolMetadata)
        metadata_path = get_cwl_tool_metadata('STAR', '2.5', subtool_name='alignReads')
        validate_subtool_metadata(metadata_path)
        return

    def test_validate_subtool_metadata_fail(self):
        validate_subtool_metadata = metadata_validator_factory(SubtoolMetadata)
        metadata_path = Path.cwd() / 'tests/test_files/test_content/cwl_tools_bad/samtools/1.3/samtools_notRight/samtools-notRight-metadata.yaml'  # subtool name not in parent featurelist.
        with self.assertRaises(ValueError):
            validate_subtool_metadata(metadata_path)
        return






# Classes to represent metadata for command line tools.
from pathlib import Path
from collections import OrderedDict
from abc import abstractmethod
import re
from ruamel.yaml import safe_load
from ...classes.metadata.metadata_base import MetadataBase
from ...classes.metadata.shared_properties import CodeRepository, Person, Publication, WebSite, Keyword, ApplicationSuite
from ...helpers.get_metadata_from_biotools import make_tool_metadata_kwargs_from_biotools
from ...classes.metadata.common_functions import _mk_hashes, NameSoftwareVersionMixin


class ToolMetadataBase(MetadataBase):
    """Factor stuff out to here."""

    @abstractmethod
    def _mk_identifier(self, **kwargs):
        pass

    @abstractmethod
    def _check_identifier(self, identifier):
        pass

    @property
    def identifier(self):
        return self._identifier

    @identifier.setter
    def identifier(self, identifier=None, **kwargs):
        if identifier:
            identifier = self._check_identifier(identifier)
        else:
            identifier = self._mk_identifier(**kwargs)
        self._identifier = identifier

    @property
    def keywords(self):
        return self._keywords

    @keywords.setter
    def keywords(self, keywords_list):
        if keywords_list:
            keywords = []
            for keyword in keywords_list:
                if isinstance(keyword, Keyword):
                    keywords.append(keyword)
                else:
                    if isinstance(keyword, dict):
                        keywords.append(Keyword(**keyword))
                    else:
                        keywords.append(Keyword(keyword))
        else:
            keywords = [Keyword()]
        self._keywords = keywords


class ToolMetadata(NameSoftwareVersionMixin, ToolMetadataBase):
    """Class to represent metadata for a 'stand alone' command line tool."""

    @staticmethod
    def _init_metadata():
        return OrderedDict([
            ('name', None),
            ('softwareVersion', None),
            ('version', '0.1.0'),  # Set to something low if not provided.
            ('identifier', None),
            ('description', None),
            ('codeRepository', CodeRepository()),
            ('license', None),
            ('WebSite', [WebSite()]),
            ('contactPoint', [Person()]),
            ('publication', [Publication()]),
            ('keywords', None),
            ('alternateName', None),
            ('creator', [Person()]),
            ('programmingLanguage', None),
            ('datePublished', None),
            ('downloadURL', None),
            ('extra', None),
        ])

    def _check_identifier(self, identifier):
        if not identifier[:3] == "TL_":
            raise ValueError(f"Tool identifiers must start with 'TL_' you provided {identifier}")
        else:
            hex_pattern = r'[0-9a-f]{6}\.[0-9a-f]{2}$'
            match_obj = re.match(hex_pattern, identifier[3:])
            if not match_obj:
                raise ValueError(f"Tool identifier not formatted correctly: {identifier}")

        return identifier

    def _mk_identifier(self, start=0):
        if not (self.name and self.softwareVersion):
            raise ValueError(f"Name and softwareVersion must be provided to make an identifier.")
        name_hash, version_hash = _mk_hashes(self.name, self.softwareVersion)
        identifier = f"TL_{name_hash[start:start + 6]}.{version_hash[:2]}"
        return identifier

    # Class factory methods

    @classmethod
    def load_from_file(cls, file_path):
        file_path = Path(file_path)
        with file_path.open('r') as file:
            file_dict = safe_load(file)
        return cls(**file_dict)

    @classmethod
    def create_from_biotools(cls, biotools_id, softwareVersion, version='0.1.1'):
        kwargs = make_tool_metadata_kwargs_from_biotools(biotools_id)
        kwargs['softwareVersion'] = softwareVersion
        kwargs['version'] = version
        return cls(**kwargs)

    def mk_instance(self):
        raise NotImplementedError


class ParentToolMetadata(ToolMetadataBase):

    @staticmethod
    def _init_metadata():
        return OrderedDict([
            ('name', None),
            ('softwareVersion', None),
            ('version', '0.1.0'),
            ('identifier', None),
            ('featureList', None),
            ('description', None),
            ('codeRepository', CodeRepository()),
            ('license', None),
            ('WebSite', [WebSite()]),
            ('contactPoint', [Person()]),
            ('publication', [Publication()]),
            ('keywords', None),
            ('alternateName', None),
            ('creator', [Person()]),
            ('programmingLanguage', None),
            ('datePublished', None),
            ('downloadURL', None),
            ('extra', None)
        ])

    def _check_identifier(self, identifier):
        if not identifier[:3] == "TL_":
            raise ValueError(f"Tool identifiers must start with 'TL_' you provided {identifier}")
        else:
            hex_pattern = r'[0-9a-f]{6}\.[0-9a-f]{2}$'
            match_obj = re.match(hex_pattern, identifier[3:])
            if not match_obj:
                raise ValueError(f"Tool identifier not formatted correctly: {identifier}")

        return identifier

    def _mk_identifier(self, start=0):
        if not (self.name and self.softwareVersion):
            raise ValueError(f"Name and softwareVersion must be provided to make an identifier.")
        name_hash, version_hash = _mk_hashes(self.name, self.softwareVersion)
        identifier = f"TL_{name_hash[start:start + 6]}.{version_hash[:2]}"
        return identifier

    def make_subtool_metadata(self, subtool_name, parent_metadata_path):
        if not self.featureList:
            raise ValueError(f"Cannot create subtool. featureList of {self.name} is not populated.")
        if subtool_name not in self.featureList:
            raise ValueError(f"{subtool_name} must be in the parent featureList")
        subtool_metadata = SubtoolMetadata(name=subtool_name, _parentMetadata=self, parentMetadata=parent_metadata_path)
        return subtool_metadata

    @classmethod
    def load_from_file(cls, file_path):
        file_path = Path(file_path)
        with file_path.open('r') as file:
            file_dict = safe_load(file)
        return cls(**file_dict)

    @classmethod
    def create_from_biotools(cls, biotools_id, softwareVersion, subtools, version='0.1.1'):
        kwargs = make_tool_metadata_kwargs_from_biotools(biotools_id)
        kwargs['featureList'] = list(subtools)
        kwargs['softwareVersion'] = softwareVersion
        kwargs['version'] = version
        return cls(**kwargs)


class SubtoolMetadata(ToolMetadataBase):

    @staticmethod
    def _init_metadata():
        return OrderedDict([
            ('applicationSuite', ApplicationSuite()),
            ('name', None),
            ('version', '0.1'),
            ('identifier', None),
            ('description', None),
            ('keywords', None),
            ('alternateName', None),
            ('parentMetadata', None),  # relative path to parentMetadata
            ('_parentMetadata', None),  # ParentMetadata instance. Can be loaded from parentMetadata or set directly.
            ('_primary_file_attrs', None),
        ])

    def __init__(self, file_path=None, **kwargs):
        # file_path only used if class is initiated using 'load_from_file'. file_path is path that SubtoolMetadata is loaded from.
        self._parentMetadata = kwargs.get('_parentMetadata')
        if not self._parentMetadata:
            self.parentMetadata = kwargs['parentMetadata']  # must have a path if it isn't set directly.
            self._load_parent_metadata(file_path)  # sets self._parentMetadata
        self._primary_file_attrs = []
        for k, value in kwargs.items():
            if value:
                self._primary_file_attrs.append(k)  # keep track of kwargs supplied.
        self._load_attrs_from_parent()
        super().__init__(**kwargs)


    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, value):
        if not value in self._parentMetadata.featureList:
            raise ValueError(f"{value} is not in {self._parentMetadata.name} metadata featureList")
        self._name = value


    @classmethod
    def load_from_file(cls, file_path):
        file_path = Path(file_path)
        with file_path.open('r') as file:
            file_dict = safe_load(file)
        return cls(**file_dict, file_path=file_path)

    def _load_parent_metadata(self, subtool_metadata_file_path):
        # subtool_metadata_file_path of subtool metadata. ParentMetadata path should be relative to this.
        dir_name = subtool_metadata_file_path.parent
        full_path = dir_name / self.parentMetadata
        with full_path.resolve().open('r') as f:
            parent_metadata_dict = safe_load(f)
        self._parentMetadata = ParentToolMetadata(**parent_metadata_dict)

    def _load_attrs_from_parent(self):
        # initialize everything from parent. Will be overwritten anything supplied in kwargs.
        parent_meta = self._parentMetadata
        self.applicationSuite = ApplicationSuite(name=parent_meta.name, softwareVersion=parent_meta.softwareVersion,
                                                 identifier=parent_meta.identifier)
        # self.identifier = self._mk_identifier()
        self.keywords = parent_meta.keywords
        return

    def _mk_identifier(self):
        identifier_str, version_str = self._parentMetadata.identifier.split('.', 1)
        subtool_hash = _mk_hashes(self.name)
        identifier = f"{identifier_str}_{subtool_hash}.{version_str}"
        return identifier

    def _check_identifier(self, identifier):
        parent_identifier = self._parentMetadata.identifier
        if not identifier.startswith(parent_identifier[:9]):
            raise ValueError(f"Subtool identifier {identifier} does not properly correspond to parent identifier {parent_identifier}")
        if not identifier.endswith(parent_identifier[-3:]):  # should be '.xx'
            raise ValueError(
                f"Subtool identifier {identifier} does not properly correspond to parent identifier {parent_identifier}")
        hex_pattern = r'[0-9a-f]{6}_[0-9a-f]{2}\.[0-9a-f]{2}$'
        match_obj = re.match(hex_pattern, identifier[3:])
        if not match_obj:
            raise ValueError(f"Tool identifier not formatted correctly: {identifier}")

        return identifier

    @classmethod
    def initialize_from_parent(cls, parent_metadata, subtool_name):
        subtool_dict = {}
        subtool_dict['applicationSuite'] = {'name': parent_metadata.name,
                                            'softwareVersion': parent_metadata.softwareVersion,
                                            'identifier': parent_metadata.identifier}
        if not subtool_name in parent_metadata.featureList:
            raise ValueError(
                f"Cannot create subtool metadata. {subtool_name} is not in featureList of {parent_metadata.name}")
        subtool_dict['name'] = subtool_name
        return cls(**subtool_dict)

    def mk_completed_file(self):
        raise NotImplementedError

    def mk_instance(self):
        raise NotImplementedError

    def mk_file(self, file_path, keys=None):
        parent_path = Path(self.parentMetadata)
        subtool_path = file_path
        rel_path = parent_path.relative_to(subtool_path.parents[1])
        rel_path_str = '../' + str(rel_path)
        self.parentMetadata = rel_path_str
        super().mk_file(file_path)




from pathlib import Path
from src.classes.metadata.workflow_metadata import WorkflowMetadata


def _get_workflow_directory(group_name, workflow_name, workflow_version):
    """
    Gets the script directory specified. Makes the directory if it doesn't exist.
    :param group_name:
    :param project_name:
    :param script_version:
    :return:
    """
    base_path = Path().cwd() / 'cwl-workflows' / group_name / workflow_name / workflow_version
    if base_path.exists():
        if not base_path.is_dir():
            raise TypeError(f"{base_path} must be a directory")
    else:
        base_path.mkdir(parents=True)
    return base_path


def add_workflow(group_name, workflow_name, workflow_version, **kwargs ):
    workflow_version = str(workflow_version)
    wf_path = _get_workflow_directory(group_name, workflow_name, workflow_version)
    wf_metadata = WorkflowMetadata(name=workflow_name, softwareVersion=workflow_version, **kwargs)
    filename = f"{workflow_name}-metadata.yaml"
    wf_metadata.mk_file(wf_path / filename)
    instances_dir = wf_path / 'instances'
    instances_dir.mkdir()
    return


import os
from tests.test_base import TestBase
from ruamel.yaml import safe_load


class TestMetadataFromBioTools(TestBase):

    class_data = {'biotoolsID': 'signalP',
                  'name': 'SignalP'}

    def test_populate_file(self):
        pass
        # biotools_ID = TestMetadataFromBioTools.class_data['biotoolsID']
        # file_name = make_metadata_file_from_biotools(biotools_ID)
        # with open(file_name, 'r') as file:
        #     meta_file_dict = safe_load(file)
        # self.assertEqual(meta_file_dict['name'], TestMetadataFromBioTools.class_data['name'])
        # os.remove(file_name)



#
# * This file is subject to the terms and conditions defined in
# * file 'LICENSE.txt', which is part of this source code package.

import os
from pathlib import Path
from ruamel.yaml import safe_load
from tests.test_base import TestBase
from xd_cwl_utils.config import config
from xd_cwl_utils.helpers.get_paths import get_workflow_metadata
from xd_cwl_utils.classes.metadata.workflow_metadata import WorkflowMetadata


class TestWorkflowMetadata(TestBase):

    def test_make_workflow_metadata(self):
        test_name = 'Test wf name'
        wf = WorkflowMetadata(name=test_name, softwareVersion=1)
        self.assertEqual(test_name, wf.name)

    def test_load_from_file(self):
        metafile_path = get_workflow_metadata('example_workflows', 'cat_sort', '1.0', 'cat_sort')
        wf_meta = WorkflowMetadata.load_from_file(metafile_path)
        self.assertEqual(wf_meta.name, 'cat sort')

    def test_mk_file(self):
        metafile_path = get_workflow_metadata('example_workflows', 'cat_sort', '1.0', 'cat_sort')
        wf_meta = WorkflowMetadata.load_from_file(metafile_path)
        test_filename = Path(config[os.environ.get('CONFIG_KEY')]['temp_dir'].name) / 'wf_test_metadata.yaml'
        wf_meta.mk_file(test_filename)
        with test_filename.open('r') as file:
            test_file_dict = safe_load(file)
        self.assertEqual(test_file_dict['identifier'], 'WF_1f4d8f.cb')
        os.remove(test_filename)


import tempfile
from unittest import TestCase
from pathlib import Path


class TestBase(TestCase):

    @staticmethod
    def get_metadata_path(rel_primary_file_path):
        full_path = Path.cwd() / rel_primary_file_path
        path_dir = full_path.parent
        file_name = full_path.stem
        metafile_name = file_name + '-metadata.yaml'
        metafile_path = path_dir / metafile_name
        return metafile_path



    def setUp(self):
        self.test_dir = tempfile.TemporaryDirectory()

    def tearDown(self) -> None:
        pass


#
# * This file is subject to the terms and conditions defined in
# * file 'LICENSE.md', which is part of this source code package.

from abc import abstractmethod, ABC
from urllib.parse import urlparse
from ruamel.yaml.comments import CommentedMap

class AttributeBase(ABC):

    def __init__(self, *args, **kwargs):
        __slots__ = list(self.attrs)

    def is_empty(self):
        _is_empty = True
        for attribute in self.attrs:
            if isinstance(attribute, list):
                for item in attribute:
                    if getattr(item, 'attrs'):
                        if not item.is_empty():
                            _is_empty = False
                            break
                    elif item:
                        _is_empty = False
                        break
            elif getattr(self, attribute):
                _is_empty = False
                break
            else:
                pass
        return _is_empty

    @staticmethod
    @abstractmethod
    def _attrs():
        return frozenset([])


    @property
    def attrs(self):
        return self._attrs()

    def dump(self):
        map_object = CommentedMap()
        for attribute in self.attrs:
            map_object[attribute] = getattr(self, attribute)
        return map_object


class CodeRepository(AttributeBase):
    def __init__(self, name=None, URL=None):
        super().__init__()
        self._name = name
        self._URL = URL
        return

    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, new_name):
        self._name = new_name
        return

    @property
    def URL(self):
        return self._URL

    @URL.setter
    def URL(self, new_URL):
        if new_URL:
            valid_schemes = ['https', 'http', 'git']
            parse_result = urlparse(new_URL)
            if parse_result.scheme not in valid_schemes:
                raise ValueError(f"URL scheme should be in {valid_schemes}")
        else:
            new_URL = None
        self._URL = new_URL
        return


    @staticmethod
    def _attrs():
        return frozenset(['name', 'URL'])


class WebSite(AttributeBase):
    def __init__(self, name=None, description=None, URL=None):
        super().__init__()
        self._name = name
        self._description = description
        self._URL = URL

    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, new_name):
        self._name = new_name

    @property
    def description(self):
        return self._description

    @description.setter
    def description(self, new_description):
        self._description = new_description

    @property
    def URL(self):
        return self._URL

    @URL.setter
    def URL(self, new_URL):
        if new_URL:
            valid_schemes = ['https', 'http']
            parse_result = urlparse(new_URL)
            if parse_result.scheme not in valid_schemes:
                raise ValueError(f"URL scheme should be in {valid_schemes}")
        else:
            new_URL = None
        self._URL = new_URL
        return

    @staticmethod
    def _attrs():
        return frozenset(['name', 'description', 'URL'])


class Publication(AttributeBase):
    def __init__(self, identifier=None, headline=None):
        super().__init__()
        self._identifier = identifier
        self._headline = headline
        return

    @property
    def identifier(self):
        return self._identifier

    @identifier.setter
    def identifier(self, new_identifier):
        self._identifier = new_identifier
        return

    @property
    def headline(self):
        return self._headline

    @headline.setter
    def headline(self, new_headline):
        self._headline = new_headline
        return

    @staticmethod
    def _attrs():
        return frozenset(['identifier', 'headline'])


class Person(AttributeBase):
    def __init__(self, name=None, email=None, identifier=None):
        super().__init__()
        self._name = name
        self._email = email
        self._identifier = identifier

    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, new_name):
        self._name = new_name

    @property
    def email(self):
        return self._email

    @email.setter
    def email(self, new_email):
        self._email = new_email
        return

    @property
    def identifier(self):
        return self._identifier

    @identifier.setter
    def identifier(self, new_identifier):
        self._identifier = new_identifier

    @staticmethod
    def _attrs():
        return frozenset(['name', 'email', 'identifier'])


class Keyword(AttributeBase):
    def __init__(self, *args, **kwargs):  # Need to initialize off of *args and **kwargs to handle both forms.
        super().__init__()
        args_len = len(args)
        if args_len == 0:
            self._uri = kwargs.get('uri', None)
        elif args_len == 1:
            self._uri = args[0]
        else:
            raise ValueError(f"Expected only one argument for uri for keyword. Got {args}")
        self._name = kwargs.get('name', None)
        self._category = kwargs.get('category', None)

    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, new_name):
        self._name = new_name

    @property
    def category(self):
        return self._category

    @category.setter
    def category(self, new_category):
        category_values = (None, 'topic', 'operation')
        if new_category not in category_values:
            raise ValueError(f"{new_category} is not a valid category for a keyword. Must be one of {category_values}")
        self._category = new_category

    @property
    def uri(self):
        return self._uri

    @uri.setter
    def uri(self, new_uri):
        self._uri = new_uri

    def dump(self):
        if self.uri:
            return self.uri
        else:
            keyword = CommentedMap([('name', self.name), ('category', self.category)])
            return keyword

    @staticmethod
    def _attrs():
        return frozenset(['name', 'category', 'uri'])

class ApplicationSuite(AttributeBase):

    def __init__(self, name=None, softwareVersion=None, identifier=None):
        super().__init__()
        self._name = name
        self._softwareVersion = softwareVersion
        self._identifier = identifier

    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, new_name):
        self._name = new_name

    @property
    def softwareVersion(self):
        return self._softwareVersion

    @softwareVersion.setter
    def softwareVersion(self, new_softwareVersion):
        self._softwareVersion = new_softwareVersion

    @property
    def identifier(self):
        return self._identifier

    @identifier.setter
    def identifier(self, new_identifier):
        self._identifier = new_identifier


    @staticmethod
    def _attrs():
        return frozenset(['name', 'softwareVersion', 'identifier'])


class ParentScript(AttributeBase):

    def __init__(self, name=None, softwareVersion=None, identifier=None):
        super().__init__()
        self._name = name
        self._softwareVersion = softwareVersion
        self._identifier = identifier

    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, new_name):
        self._name = new_name


    @property
    def softwareVersion(self):
        return self._softwareVersion

    @softwareVersion.setter
    def softwareVersion(self, new_softwareVersion):
        self._softwareVersion = new_softwareVersion

    @property
    def identifier(self):
        return self._identifier

    @identifier.setter
    def identifier(self, new_identifier):
        self._identifier = new_identifier

    @staticmethod
    def _attrs():
        return frozenset(['name','softwareVersion', 'identifier'])


class Tool(AttributeBase):

    def __init__(self, name=None, softwareVersion=None, identifier=None, alternateName=None):
        super().__init__()
        self._name = name
        self._alternateName = alternateName
        self._softwareVersion = softwareVersion
        self._identifier = identifier

    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, new_name):
        self._name = new_name

    @property
    def alternateName(self):
        return self._alternateName

    @alternateName.setter
    def alternateName(self, value):
        self._alternateName = value

    @property
    def softwareVersion(self):
        return self._softwareVersion

    @softwareVersion.setter
    def softwareVersion(self, new_softwareVersion):
        self._softwareVersion = new_softwareVersion

    @property
    def identifier(self):
        return self._identifier

    @identifier.setter
    def identifier(self, new_identifier):
        self._identifier = new_identifier


    @staticmethod
    def _attrs():
        return frozenset(['name', 'alternateName', 'softwareVersion', 'identifier'])


class CallMap(AttributeBase):

    def __init__(self, id_=None, identifier=None):
        super().__init__()

        self._id = id_
        self._identifier = identifier

    @property
    def id(self):
        return self._id

    @id.setter
    def id(self, new_id):
        self._id = new_id

    @property
    def identifier(self):
        return self._identifier

    @identifier.setter
    def identifier(self, new_identifier):
        self._identifier = new_identifier

    @staticmethod
    def _attrs():
        return frozenset(['id', 'identifier'])


class IOObject(AttributeBase):

    def __init__(self, identifier=None, path=None):
        super().__init__()
        self._identifier = identifier
        self._path = path

    @property
    def identifier(self):
        return self._identifier

    @identifier.setter
    def identifier(self, new_identifier):
        self._identifier = new_identifier

    @property
    def path(self):
        return self._path

    @path.setter
    def path(self, new_path):
        self._path = new_path


    @staticmethod
    def _attrs():
        return frozenset(['identifier', 'path'])

class IOObjectItem(AttributeBase):
    def __init__(self, id_=None, io_object=IOObject()):
        super().__init__()

        self._id = id_
        self._io_object = io_object

    @property
    def id(self):
        return self._id

    @id.setter
    def id(self, new_id):
        self._id = new_id

    @property
    def identifier(self):
        return self._io_object._identifier

    @identifier.setter
    def identifier(self, new_identifier):
        self._io_object._identifier = new_identifier

    @property
    def path(self):
        return self._io_object._path

    @path.setter
    def path(self, new_path):
        self._io_object._path = new_path

    @property
    def io_object(self):
        return self._io_object

    @staticmethod
    def _attrs():
        return frozenset(['id', 'io_object'])

    def dump(self):
        map_object = CommentedMap()
        map_object['id'] = getattr(self, 'id')
        map_object.update(self.io_object.dump())
        return map_object

class IOArrayItem(AttributeBase):
    def __init__(self, id_, objects=None):
        super().__init__()
        self._id = id_
        self._objects = objects if objects else [IOObject()]

    @staticmethod
    def _attrs():
        return frozenset(['id', 'objects'])







#
# * This file is subject to the terms and conditions defined in
# * file 'LICENSE.txt', which is part of this source code package.

import os
from pathlib import Path
from ruamel.yaml import safe_load
from xd_cwl_utils.config import config
from tests.test_base import TestBase
from xd_cwl_utils.classes.metadata.script_metadata import ScriptMetadata
from xd_cwl_utils.helpers.get_paths import get_cwl_script, get_metadata_path

class TestScriptMetadata(TestBase):

    def test_make_script_metadata_from_kwargs(self):
        kwargs = {'name': 'test_script', 'softwareVersion': 1, 'identifier': 'ST_abcdef.12'}
        st_metadata = ScriptMetadata(**kwargs)
        self.assertEqual(st_metadata.name, kwargs['name'])
        self.assertEqual(st_metadata.identifier, kwargs['identifier'])
        return

    def test_mk_file_from_script_metadata(self):
        kwargs = {'name': 'test_script', 'softwareVersion': 1, 'identifier': 'ST_abcdef.12'}
        st_metadata = ScriptMetadata(**kwargs)
        test_filename = Path(config[os.environ.get('CONFIG_KEY')]['temp_dir'].name) / 'script_test_metadata.yaml'
        st_metadata.mk_file(test_filename)
        with test_filename.open('r') as file:
            test_file_dict = safe_load(file)
        self.assertEqual(test_file_dict['identifier'], kwargs['identifier'])
        os.remove(test_filename)

    def test_make_script_metadata_from_file(self):
        script_path = get_cwl_script('ENCODE-DCC', 'atac-seq-pipeline','v1.1', 'encode_ataqc')
        metadata_path = get_metadata_path(script_path)
        st_metadata = ScriptMetadata.load_from_file(metadata_path)
        return

    # @skip("Pass")
    def test_mk_file_with_inherited_data(self):
        script_cwl_path = get_cwl_script('ENCODE-DCC', 'atac-seq-pipeline', 'v1.1', 'encode_ataqc')
        metadata_path = get_metadata_path(script_cwl_path)
        st_metadata = ScriptMetadata.load_from_file(metadata_path)
        test_filename = Path(config[os.environ.get('CONFIG_KEY')]['temp_dir'].name) / 'script2_test_metadata.yaml'
        st_metadata.mk_completed_file(test_filename)
        with test_filename.open('r') as file:
            test_file_dict = safe_load(file)
        self.assertEqual(test_file_dict['license'], 'MIT')
        os.remove(test_filename)
        return

    def test_mk_file_without_inherited_data(self):
        script_cwl_path = get_cwl_script('ENCODE-DCC', 'atac-seq-pipeline', 'v1.1', 'encode_ataqc')
        metadata_path = get_metadata_path(script_cwl_path)
        st_metadata = ScriptMetadata.load_from_file(metadata_path)
        test_filename = Path(config[os.environ.get('CONFIG_KEY')]['temp_dir'].name) / 'script3_test_metadata.yaml'
        st_metadata.mk_file(test_filename)
        with test_filename.open('r') as file:
            test_file_dict = safe_load(file)
        with self.assertRaises(KeyError):
            license = test_file_dict['license']
        os.remove(test_filename)
        return


import os
from tests.test_base import TestBase
from pathlib import Path
from ruamel.yaml import safe_load
from xd_cwl_utils.config import config
from xd_cwl_utils.helpers.get_paths import get_metadata_path
from xd_cwl_utils.validate_metadata import main

class TestValidateContent(TestBase):

    def test_validate_tools(self):
        tool_map = config[os.environ.get('CONFIG_KEY')]['content_maps_dir'] / 'tool-maps.yaml'
        with tool_map.open('r') as tm:
            tool_map_dict = safe_load(tm)
        for identifier, values in tool_map_dict.items():
            path = Path(values['path'])
            tool_type = values['type']
            if tool_type == 'parent':
                if not 'common' in path.parts:  # values[type] would be better test.
                    raise ValueError(f"Have a parent tool that is not in a common directory {path}")
                meta_type = 'parent_tool'
                meta_path = path
            else:  # either a subtool or standalone tool.
                meta_path = get_metadata_path(path)
                meta_type = tool_type
            main([meta_type, str(meta_path)])
        return

    def test_validate_scripts(self):
        script_map_path = config[os.environ.get('CONFIG_KEY')]['content_maps_dir'] / 'script-maps.yaml'
        with script_map_path.open('r') as sm:
            script_map = safe_load(sm)
        for script_identifier, script_values in script_map.items():
            metadata_path = get_metadata_path(Path(script_values['path']))
            main(['script', str(metadata_path)])
        return


    def test_validate_workflows(self):
        workflow_maps_path =config[os.environ.get('CONFIG_KEY')]['content_maps_dir'] / 'workflow-maps.yaml'
        with workflow_maps_path.open('r') as wm:
            workflow_map = safe_load(wm)
        for workflow_identifier, workflow_values in workflow_map.items():
            metadata_path = get_metadata_path(workflow_values['path'])
            main(['workflow', str(metadata_path)])
        return


#
# * This file is subject to the terms and conditions defined in
# * file 'LICENSE.txt', which is part of this source code package.


from abc import ABC, abstractmethod
import logging
from pathlib import Path
import semantic_version
from ruamel.yaml.comments import CommentedMap
from ruamel.yaml import YAML
from ...classes.metadata.shared_properties import CodeRepository, Person, Publication, WebSite, Keyword, ApplicationSuite, ParentScript, Tool, IOObjectItem, CallMap

object_attributes = (CodeRepository, Person, Publication, WebSite, Keyword, ApplicationSuite, ParentScript, Tool, IOObjectItem, CallMap)

class MetadataBase(ABC):
    """Factor stuff out to here."""

    @staticmethod
    @abstractmethod
    def _init_metadata():
        return dict([('name', None), ('version', None)])

    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, new_name):
        # Can put validators here.
        self._name = new_name


    @property
    def version(self):
        return self._version

    @version.setter
    def version(self, new_version):
        new_version = str(new_version)
        is_semantic = semantic_version.validate(new_version)
        if is_semantic:
            v = semantic_version.Version(new_version)
        else:
            logging.info(f"'{new_version}' is not a properly formatted semantic version")
            try:
                v = semantic_version.Version(new_version, partial=True)
            except ValueError:
                raise ValueError(f"'{new_version}'' is not a valid partial semantic version.")
            v = semantic_version.Version.coerce(str(v))
        self._version = str(v)



    def _get_metafile_keys(self):
        return list(self._init_metadata())


    def __init__(self, **kwargs):
        init_metadata = self._init_metadata()

        for k, v in kwargs.items():
            if not k in init_metadata:
                raise AttributeError(f"{k} is not a valid key for {type(self)}")

        for k, v in init_metadata.items():
            if k in kwargs:
                setattr(self, k, kwargs[k])  # Highest priority.
            else:
                try:
                    if getattr(self, k):  # value has already been set by derived class __init__. Second highest priority.
                        continue
                    else:
                        raise NotImplementedError(f"Figure out what's happening here and fix it.")
                except AttributeError:
                    setattr(self, k, v)  # Set to default value provided in self._init_metadata. Last resort.
        return

    def mk_file(self, file_path, keys=None):
        file_path = Path(file_path)
        meta_map = CommentedMap()
        if not keys:
            keys = self._get_metafile_keys()
        for key in keys:
            if key.startswith('_'):
                continue
            attr_value = getattr(self, key)
            if isinstance(attr_value, object_attributes):
                meta_map[key] = attr_value.dump()
            elif isinstance(attr_value, list):
                if isinstance(attr_value[0], object_attributes):
                    meta_map[key] = [item.dump() for item in attr_value]
                else:
                    meta_map[key] = attr_value
            else:
                meta_map[key] = attr_value
        yaml = YAML()
        yaml.default_flow_style = False
        yaml.indent(mapping=2, sequence=4, offset=2)
        with file_path.open('w') as yaml_file:
            yaml.dump(meta_map, yaml_file)
        return



from cwltool.process import shortname

class CommandLineToolMixin:
    """Mixin methods for working with cwl_classes.CommandLineTool objects. These objects should be preprocessed and
    validated before using these methods"""

    cwl_types = ('null', 'boolean', 'int', 'long', 'float', 'double', 'string', 'File', 'Directory')

    def _get_schema_def_requirement(self):
        """
        Checks for SchemaDefRequirment in requirements, and if it is there returns it.
        :return: SchemaDefRequirement | None
        """
        schema_def_requirement = None
        if self.requirements:
            for requirement in self.requirements:
                try:
                    requirement.__getattribute__('types')
                    schema_def_requirement = requirement
                except AttributeError:
                    pass # Requirement is not SchemaDefRequirement. Ignore it.
        return schema_def_requirement

class CommandInputParameterMixin:

    def _handle_str_input_type(self, _type, schema_def_requirement):
        """
        Should be the terminating function of walking CommandInputParameter.type fields. Return type if type is CWLType,
        otherwise return type from SchemaDefRequirement dict.
        :param _type:
        :param schema_def_dict:
        :return:
        """
        if _type in ('null', 'boolean', 'int', 'long', 'float', 'double', 'string', 'File', 'Directory'):
            _type = _type
        else:
            schema_def_dict = schema_def_requirement._make_schema_def_dict()
            schema_def_name = shortname(_type)
            _type = schema_def_dict[schema_def_name]
        return _type

    def _handle_input_type_field(self, type_field, schema_def_requirement):
        if isinstance(type_field, str):
            input_type = self._handle_str_input_type(type_field, schema_def_requirement)

        elif isinstance(type_field, list):  # got a list of types.
            input_type = []
            for _type in type_field:
                if isinstance(_type, str):
                    input_type.append(self._handle_str_input_type(_type, schema_def_requirement))
                else:  # Need to handle record, enum, and array types here.
                    input_type.append(_type.save())
        else:
            input_type = type_field.save()  # Takes care of CommandInput[Array/Enum/Record]Schmema. Can deal with separately later if we want to.

        return input_type

class SchemaDefRequirementMixin:

    def _make_schema_def_dict(self):
        """
        Make dictionary from SchemaDefRequirement to populate inputs parameters fully.

        :param schema_def_requirement: SchemaDefRequirement object.
        :return: keys are input names, values are input parameter fields to drop into inputs section.
        :rtype: dict
        """
        schema_def_types = self.types  # list of InputRecordSchema | InputArraySchema | InputEnumSchema objects.
        if not isinstance(schema_def_types, list):
            raise NotImplementedError
        schema_def_dict = {}
        for type_def in schema_def_types:
            type_def_name = shortname(type_def.name)
            schema_def_dict[type_def_name] = type_def.save()
            schema_def_dict[type_def_name]['name'] = shortname(schema_def_dict[type_def_name]['name'])
            if type_def.type == 'record':
                pass
            elif type_def.type == 'array':
                pass
            elif type_def.type == 'enum':
                raise NotImplementedError("EnumRecordSchema puts long id's in for symbols. Make sure this is handled.")
            else:
                raise ValueError(f"Unexpected InputSchema type {repr(type_def)}")
        return schema_def_dict


import configparser
import tempfile
from pathlib import Path

test_rel_content_path = Path.cwd() / 'tests' / 'test_files' / 'test_content'
content_base_path = Path.cwd()

# Default values for using tool in repos.
_default = {
    'cwl_tool_dir': content_base_path / 'cwl-tools',
    'cwl_script_dir': content_base_path / 'cwl-scripts',
    'cwl_workflows_dir': test_rel_content_path / 'cwl-workflows',
    'content_maps_dir': Path.cwd() / 'content_maps',
    'temp_dir': tempfile.TemporaryDirectory(prefix='cwlTest_')
}

# Test values for running unit tests.
_test = {
    'cwl_tool_dir': test_rel_content_path / 'cwl-tools',
    'cwl_script_dir': test_rel_content_path / 'cwl-scripts',
    'cwl_workflows_dir': test_rel_content_path / 'cwl-workflows',
    'content_maps_dir': test_rel_content_path / 'content_maps',
    'temp_dir': tempfile.TemporaryDirectory(prefix='cwlTest_'),
}


config = {'TEST': _test, 'DEFAULT': _default}


#!/usr/bin/env python3

import argparse
import sys
import logging
from xd_cwl_utils.classes.metadata.tool_metadata import ToolMetadata, ParentToolMetadata, SubtoolMetadata
from xd_cwl_utils.classes.metadata.script_metadata import ScriptMetadata, CommonScriptMetadata
from xd_cwl_utils.classes.metadata.workflow_metadata import WorkflowMetadata


# parser = argparse.ArgumentParser(description="Validate metadata files.")
# subparsers = parser.add_subparsers(description="Specify type of metadata to validate.", dest='command')
#
# validate_tool = subparsers.add_parser('tool', help="Validate tool metadata.")
# validate_tool.add_argument('path', help="Path to tool metadata file.")
#
# validate_parent_tool = subparsers.add_parser('parent_tool', help="Validate parent tool metadata.")
# validate_parent_tool.add_argument('path', help="Path to parent tool metadata file.")
#
# validate_subtool = subparsers.add_parser('subtool', help="Validate subtool metadata.")
# validate_subtool.add_argument('path', help="Path to subtool metadata file.")
#
# validate_script = subparsers.add_parser('script', help="Validate script metadata")
# validate_script.add_argument('path', help="Path to script metadata file.")
#
# validate_script_common = subparsers.add_parser('script_common', help="Validate common script metadata.")
# validate_script_common.add_argument('path', help="Validate common script metadata")
#
# validate_workflow = subparsers.add_parser('workflow', help="Validate workflow metadata.")
# validate_workflow.add_argument('path', help="Path to workflow metadata")

def metadata_validator_factory(class_to_validate):
    def metadata_validator(metadata_path):
        try:
            metadata_instance = class_to_validate.load_from_file(metadata_path)
            # print(f"Metadata in {metadata_path} is valid {str(class_to_validate)}")
            logging.info(f"Metadata in {metadata_path} is valid {str(class_to_validate)}")
        except:
            logging.error(f"{str(class_to_validate)} in {metadata_path} failed validation.")
            raise
        return
    return metadata_validator

def main(argsl=None):
    if not argsl:
        argsl = sys.argv[1:]



    parser = argparse.ArgumentParser(description="Validate metadata files.")
    subparsers = parser.add_subparsers(description="Specify type of metadata to validate.", dest='command')

    validate_tool = subparsers.add_parser('tool', help="Validate tool metadata.")
    validate_tool.add_argument('path', help="Path to tool metadata file.")

    validate_parent_tool = subparsers.add_parser('parent_tool', help="Validate parent tool metadata.")
    validate_parent_tool.add_argument('path', help="Path to parent tool metadata file.")

    validate_subtool = subparsers.add_parser('subtool', help="Validate subtool metadata.")
    validate_subtool.add_argument('path', help="Path to subtool metadata file.")

    validate_script = subparsers.add_parser('script', help="Validate script metadata")
    validate_script.add_argument('path', help="Path to script metadata file.")

    validate_script_common = subparsers.add_parser('script_common', help="Validate common script metadata.")
    validate_script_common.add_argument('path', help="Validate common script metadata")

    validate_workflow = subparsers.add_parser('workflow', help="Validate workflow metadata.")
    validate_workflow.add_argument('path', help="Path to workflow metadata")

    args = parser.parse_args(argsl)
    command = args.command
    path =args.path

    if command == 'tool':
        validate_tool_metadata = metadata_validator_factory(ToolMetadata)
        validate_tool_metadata(path)
    elif command == 'parent_tool':
        validate_parent_tool_metadata = metadata_validator_factory(ParentToolMetadata)
        validate_parent_tool_metadata(path)
    elif command == 'subtool':
        validate_subtool_metadata = metadata_validator_factory(SubtoolMetadata)
        validate_subtool_metadata(path)
    elif command == 'script':
        validate_script_metadata = metadata_validator_factory(ScriptMetadata)
        try:
            validate_script_metadata(path)
        except:
            logging.error(f"Problem with {path}")
            raise
    elif command == 'script_common':
        validate_common_script_metadata = metadata_validator_factory(CommonScriptMetadata)
        validate_common_script_metadata(path)
    elif command == 'workflow':
        validate_workflow_metadata = metadata_validator_factory(WorkflowMetadata)
        validate_workflow_metadata(path)
    else:
        parser.print_help()
    return 0

if __name__ == "__main__":
    sys.exit(main(sys.argv[1:]))



from setuptools import setup, find_packages


with open('README.md', 'r') as fh:
    long_description = fh.read()

setup(
    name='xd_cwl_utils',
    version='0.1.2',
    packages=find_packages(),
    url='https://github.com/xDBio-Inc/xd-cwl-utils',
    license='Apache 2.0',
    author='Karl Sebby',
    author_email='karl.sebby@xdbio.com',
    description='Tool for managing CWL file repositories.',
    long_description=long_description,
    long_description_content_type="text/markdown",
    install_requires = [
        'setuptools',
        'requests',
        'ruamel.yaml >= 0.15, <=0.16',
        'semantic-version',
        'pandoc-include',
        'cwltool',
    ],
    entry_points={
        'console_scripts': ["xd-cwl-validate=xd_cwl_utils.validate_metadata:main"]
                  },
    classifiers=[
        "Development Status :: 3 - Alpha",
        "License :: OSI Approved :: Apache Software License",
    ],
    python_requires=">=3",
)
